package com.xynet.marketing.controller.merc;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xynet.marketing.enums.CouponSendTypeEnum;
import com.xynet.marketing.enums.CouponTypeEnum;
import com.xynet.marketing.enums.CouponValidTypeEnum;
import com.xynet.marketing.service.MemberCouponService;
import com.xynet.marketing.service.MercCouponService;
import com.xynet.marketing.service.factory.coupon.merc.MercCouponFactory;
import com.xynet.marketing.utils.FactoryUtils;
import com.xynet.marketing.utils.MercAuthUtils;
import com.xynet.marketing.utils.R;
import com.xynet.marketing.utils.collections.map.JHashMap;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;

import java.util.Map;

/**
 * 商户端优惠券控制器
 *
 * @author hechunping
 * @date 2025/2/14
 */
@RestController
@RequestMapping("merc/coupon")
@AllArgsConstructor
@Slf4j
@CrossOrigin(origins = "*")
public class MercCouponController {
    private MercCouponService mercCouponService;

    private MemberCouponService memberCouponService;


    /**
     * 分页查询
     *
     * @param param
     * @param headers
     * @return
     */
    @PostMapping("page")
    public R page(@RequestBody JHashMap<String, Object> param, @RequestHeader Map<String, String> headers) {
        Integer mercId = MercAuthUtils.getMercId(headers);
        param.put("mercId", mercId);
        Page<Map<String, Object>> data = mercCouponService.page(param);
        return R.ok(data);
    }

    /**
     * 新增与修改
     *
     * @param param
     * @param headers
     * @return
     */
    @PostMapping("saveOrUpdate")
    public R saveOrUpdate(@RequestBody JHashMap<String, Object> param, @RequestHeader Map<String, String> headers) {
        Integer mercId = MercAuthUtils.getMercId(headers);
        param.put("mercId", mercId);
        return R.ok(mercCouponService.saveOrUpdate(param));
    }

    /**
     * 获取有效期类型枚举
     *
     * @return
     */
    @PostMapping("validTypeMaps")
    public R validTypeMaps() {
        return R.ok(CouponValidTypeEnum.maps());
    }

    /**
     * 获取优惠券类型列表
     *
     * @return
     */
    @PostMapping("typeMaps")
    public R typeMaps() {
        return R.ok(CouponTypeEnum.maps());
    }

    /**
     * 获取优惠券发放类型列表
     *
     * @return
     */
    @PostMapping("sendTypeMaps")
    public R sendTypeMaps() {
        return R.ok(CouponSendTypeEnum.maps());
    }

    /**
     * 对象查询
     *
     * @param param
     * @param headers
     * @return
     */
    @PostMapping("obj")
    public R obj(@RequestBody JHashMap<String, Object> param, @RequestHeader Map<String, String> headers) {
        Integer mercId = MercAuthUtils.getMercId(headers);
        param.put("mercId", mercId);
        return R.ok(mercCouponService.obj(param));
    }

    @PostMapping("mercSend")
    public R mercSend(@RequestBody JHashMap<String, Object> param, @RequestHeader Map<String, String> headers) {
        Integer mercId = MercAuthUtils.getMercId(headers);
        MercCouponFactory mercCouponFactory = FactoryUtils.getService(MercCouponFactory.class, "merc_coupon_base");
        param.put("mercId", mercId);
        mercCouponFactory.mercSend(param);
        return R.ok();
    }

    @PostMapping("memberCouponList")
    public R memberCouponList(@RequestBody JHashMap<String, Object> param, @RequestHeader Map<String, String> headers) {
        Integer mercId = MercAuthUtils.getMercId(headers);
        param.put("mercId", mercId);
        return R.ok(memberCouponService.page(param));
    }

    @PostMapping("updateStatus")
    public R updateEnableStatus(@RequestBody JHashMap<String, Object> param, @RequestHeader Map<String, String> headers) {
        Integer mercId = MercAuthUtils.getMercId(headers);
        param.put("mercId", mercId);
        mercCouponService.updateEnableStatus(param);
        return R.ok();
    }

}
