package com.xynet.marketing.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.xynet.marketing.entity.MemberCoupon;
import com.xynet.marketing.enums.CouponTypeEnum;
import com.xynet.marketing.enums.MemberCouponStatusEnum;
import com.xynet.marketing.mapper.MemberCouponMapper;
import com.xynet.marketing.service.MemberCouponService;
import com.xynet.marketing.service.MercProjectDeviceService;
import com.xynet.marketing.service.MercProjectGoodsService;
import com.xynet.marketing.utils.Emptys;
import com.xynet.marketing.utils.collections.map.JHashMap;
import com.xynet.marketing.utils.collections.map.JMap;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * 会员优惠券服务-实现类
 *
 * @author hechunping
 * @date 2025/2/17
 */
@Service
@Slf4j
@AllArgsConstructor
public class MemberCouponServiceImpl extends ServiceImpl<MemberCouponMapper, MemberCoupon> implements MemberCouponService {

    private MercProjectGoodsService goodsService;

    private MercProjectDeviceService deviceService;

    private String getMercId(String thirdMercId) {
        return "1";
    }

    private LambdaQueryWrapper<MemberCoupon> commonQueryWrapper(JMap<String, Object> param) {
        LambdaQueryWrapper<MemberCoupon> lqw = new LambdaQueryWrapper<MemberCoupon>()
                .eq(Emptys.check(param.getInt("id")), MemberCoupon::getId, param.getInt("id"))
                .eq(Emptys.check(param.getString("thirdMercId")), MemberCoupon::getMercId, param.getString("thirdMercId"))
                .eq(Emptys.check(param.getInt("couponId")), MemberCoupon::getCouponId, param.getInt("couponId"))
                .eq(Emptys.check(param.getString("thirdMemberId")), MemberCoupon::getThirdMemberId, param.getString("thirdMemberId"))
                .eq(Emptys.check(param.getString("status")), MemberCoupon::getStatus, param.getString("status"))
                .eq(Emptys.check(param.getString("type")), MemberCoupon::getType, param.getString("type"))
                .eq(Emptys.check(param.getString("isAllDevice")), MemberCoupon::getIsAllDevice, param.getString("isAllDevice"))
                .ge(Emptys.check(param.getString("createTimeStart")), MemberCoupon::getCreateTime, param.getString("createTimeStart"))
                .le(Emptys.check(param.getString("createTimeEnd")), MemberCoupon::getCreateTime, param.getString("createTimeEnd"))
                .ge(Emptys.check(param.getString("useTimeStart")), MemberCoupon::getUseTime, param.getString("useTimeStart"))
                .le(Emptys.check(param.getString("useTimeEnd")), MemberCoupon::getUseTime, param.getString("useTimeEnd"));


        return lqw;
    }

    /**
     * 反显信息
     *
     * @param param
     * @return
     */
    private List<Map<String, Object>> reflectName(List<Map<String, Object>> param) {
        param.forEach(item -> {
            item.put("typeName", CouponTypeEnum.getEnum(item.get("type").toString()).getValue());
            item.put("statusName", MemberCouponStatusEnum.getEnum(item.get("status").toString()).getValue());
        });
        return param;
    }


    @Override
    public Page<Map<String, Object>> page(JMap<String, Object> param) {
        LambdaQueryWrapper<MemberCoupon> lqw = commonQueryWrapper(param).orderByAsc(MemberCoupon::getId);
        Page<MemberCoupon> page = new Page<>(param.getInt("pageNum"), param.getInt("pageSize"), false);
        Page<MemberCoupon> list = page(page, lqw);
        //转换成MAP
        Page<Map<String, Object>> copy = JHashMap.copy(list);
        //反显信息
        List<Map<String, Object>> records = copy.getRecords();
        records = reflectName(records);
        //重新封装并返回
        return copy.setRecords(records);
    }

    @Override
    public List<Map<String, Object>> enableList(JMap<String, Object> param) {
        List<Map<String, Object>> result = new ArrayList<>();
        LocalDate now = LocalDate.now();
        String thirdMemberId = param.getString("thirdMemberId");
        String deviceId = param.getString("deviceId");
        Integer orderAmount = param.getInt("orderAmount");
        String thirdMercId = param.getString("thirdMercId");
        List<JMap<String, Object>> goodsList = param.getListMap("goodsList");
        LambdaQueryWrapper<MemberCoupon> lqw = new LambdaQueryWrapper<MemberCoupon>()
                .eq(MemberCoupon::getThirdMemberId, thirdMemberId)
                .eq(MemberCoupon::getMercId, getMercId(thirdMercId))
                .eq(MemberCoupon::getStatus, MemberCouponStatusEnum.UNUSED.getCode())
                .le(MemberCoupon::getUseMinPrice, orderAmount)
                .le(MemberCoupon::getValidStartDate, now)
                .ge(MemberCoupon::getValidEndDate, now);
        List<MemberCoupon> list = list(lqw);
        if (!Emptys.check(list)) {
            return result;
        }
        for (int i = 0; i < list.size(); i++) {
            MemberCoupon memberCoupon = list.get(i);
            Boolean deviceCondition = false;
            Boolean priceCondition = false;
            if (memberCoupon.getIsAllDevice()) {
                deviceCondition = true;
                //result.add(JHashMap.copy(item));
            } else {
                //判断设备是否符合
                List<Map<String, Object>> deviceList = deviceService.list(
                        new JHashMap<String, Object>()
                                .set("deviceId", deviceId)
                                .set("actId", memberCoupon.getCouponId())
                                .set("actType", "coupon")
                );
                if (Emptys.check(deviceList)) {
                    deviceCondition = true;
                    //result.add(JHashMap.copy(item));
                }
            }
            //判断商品是否符合
            if (deviceCondition) {
                if (Objects.equals(memberCoupon.getType(), CouponTypeEnum.T_1.getCode())) {
                    //满减判断价格
                    JSONArray priceJson = JSONUtil.parseArray(memberCoupon.getPrice());
                    JSONObject priceObject = priceJson.getJSONObject(0);
                    Integer limit = Integer.valueOf(priceObject.get("limit").toString());
                    if (orderAmount >= limit) {
                        priceCondition = true;
                    }
                }
            }
            if (priceCondition) {
                result.add(JHashMap.copy(memberCoupon));
            }
        }
        return result;
    }

    @Override
    public void verification(JMap<String, Object> param) {
        List<Object> memberCouponIdList = param.getList("memberCouponIdList");
        String orderId = param.getString("orderId");
        LocalDateTime orderTime = LocalDateTime.parse(param.getString("orderTime"), DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
        LambdaUpdateWrapper<MemberCoupon> luw = new LambdaUpdateWrapper<MemberCoupon>()
                .set(MemberCoupon::getStatus, MemberCouponStatusEnum.USED.getCode())
                .set(MemberCoupon::getUseTime, orderTime)
                .set(MemberCoupon::getUseOrderId, orderId)
                .in(MemberCoupon::getId, memberCouponIdList);
        update(luw);
    }

    @Override
    public Map<String, Object> obj(JMap<String, Object> param) {
        LambdaQueryWrapper<MemberCoupon> lqw = commonQueryWrapper(param).last("limit 1");
        MemberCoupon obj = getOne(lqw);
        Map<String, Object> vo = JHashMap.copy(obj);
        return vo;
    }

    @Override
    public List<Map<String, Object>> list(JMap<String, Object> param) {
        List<MemberCoupon> list = list();
        list.add(new MemberCoupon());
        return null;
    }

    @Override
    public Integer saveOrUpdate(JMap<String, Object> param) {
        MemberCoupon save = BeanUtil.copyProperties(param, MemberCoupon.class);
        if (Emptys.check(save.getId())) {
            updateById(save);
        } else {
            save(save);
        }
        return save.getId();
    }

    /**
     * 数量统计
     *
     * @param type
     * @return
     */
    @Override
    public Map<Integer, Integer> mercStatistic(List<Integer> couponIdList, String type) {
        LambdaQueryWrapper<MemberCoupon> lwq = new QueryWrapper<MemberCoupon>()
                .select("coupon_id", "count(*) as num")
                .lambda()
                .eq(!Objects.equals(type, "all"), MemberCoupon::getStatus, type)
                .in(MemberCoupon::getId, couponIdList)
                .groupBy(MemberCoupon::getCouponId);
        List<Map<String, Object>> list = listMaps(lwq);
        Map<Integer, Integer> map = list.stream().collect(
                Collectors.toMap(
                        i -> Integer.valueOf(i.get("coupon_id").toString()),
                        i -> Integer.valueOf(i.get("num").toString()))
        );
        return map;
    }
}
