package com.xynet.marketing.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.xynet.marketing.entity.MercCoupon;
import com.xynet.marketing.enums.MemberCouponStatusEnum;
import com.xynet.marketing.mapper.MercCouponMapper;
import com.xynet.marketing.service.MemberCouponService;
import com.xynet.marketing.service.MercCouponService;
import com.xynet.marketing.service.MercProjectDeviceService;
import com.xynet.marketing.service.MercProjectGoodsService;
import com.xynet.marketing.service.factory.coupon.merc.MercCouponFactory;
import com.xynet.marketing.utils.Emptys;
import com.xynet.marketing.utils.FactoryUtils;
import com.xynet.marketing.utils.collections.map.JHashMap;
import com.xynet.marketing.utils.collections.map.JMap;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * a
 * 商家优惠券服务-实现类
 *
 * @author hechunping
 * @date 2025/2/10
 */
@Service
@Slf4j
@AllArgsConstructor
public class MercCouponServiceImpl extends ServiceImpl<MercCouponMapper, MercCoupon> implements MercCouponService {

    private MercProjectGoodsService goodsService;
    private MercProjectDeviceService deviceService;

    private MemberCouponService memberCouponService;

    private static String ACT_TYPE = "coupon";

    private LambdaQueryWrapper<MercCoupon> commonQueryWrapper(JMap<String, Object> param) {
        LambdaQueryWrapper<MercCoupon> lqw = new LambdaQueryWrapper<MercCoupon>()
                .eq(Emptys.check(param.getInt("id")), MercCoupon::getId, param.getInt("id"));
        return lqw;
    }

    @Override
    public Map<String, Object> obj(JMap<String, Object> param) {
        LambdaQueryWrapper<MercCoupon> lqw = commonQueryWrapper(param).last("limit 1");
        MercCoupon obj = getOne(lqw);
        Map<String, Object> vo = JHashMap.copy(obj);
        //反显设备信息
        if (!obj.getIsAllDevice()) {
            List<Map<String, Object>> deviceList = deviceService.listByActId(obj.getId(), ACT_TYPE);
            vo.put("deviceList", deviceList);
        }
        //反显商品信息
        if (!obj.getIsAllGoods()) {
            List<Map<String, Object>> goodsList = goodsService.listByActId(obj.getId(), ACT_TYPE);
            vo.put("goodsList", goodsList);
        }
        return vo;
    }

    @Override
    public Integer saveOrUpdate(JMap<String, Object> param) {
        MercCouponFactory mercCouponFactory = FactoryUtils.getService(MercCouponFactory.class, "merc_coupon_base");
        return mercCouponFactory.saveOrUpdate(param);
    }

    @Override
    public List<Map<String, Object>> list(JMap<String, Object> param) {
        LambdaQueryWrapper<MercCoupon> lqw = commonQueryWrapper(param).orderByAsc(MercCoupon::getId);
        List<MercCoupon> list = list(lqw);
        return JHashMap.copy(list);
    }

    @Override
    public Page<Map<String, Object>> page(JMap<String, Object> param) {
        LambdaQueryWrapper<MercCoupon> lqw = commonQueryWrapper(param).orderByAsc(MercCoupon::getId);
        Page<MercCoupon> page = new Page<>(param.getInt("pageNum"), param.getInt("pageSize"), true);
        Page<MercCoupon> list = page(page, lqw);
        Page<Map<String, Object>> result = JHashMap.copy(list);
        if (Emptys.check(list.getRecords())) {
            List<Integer> idList = list.getRecords().stream().map(i -> i.getId()).collect(Collectors.toList());
            Map<Integer, Integer> allStatusNumMap = memberCouponService.mercStatistic(idList, null);
            Map<Integer, Integer> unusedStatusNumMap = memberCouponService.mercStatistic(idList, MemberCouponStatusEnum.UNUSED.getCode());
            Map<Integer, Integer> usedStatusNumMap = memberCouponService.mercStatistic(idList, MemberCouponStatusEnum.USED.getCode());
            Map<Integer, Integer> disabledStatusNumMap = memberCouponService.mercStatistic(idList, MemberCouponStatusEnum.DISABLED.getCode());
            result.getRecords().forEach(i -> {
                i.put("allStatusNum", allStatusNumMap.getOrDefault(i.get("id"), 0));
                i.put("unusedStatusNum", unusedStatusNumMap.getOrDefault(i.get("id"), 0));
                i.put("usedStatusNum", usedStatusNumMap.getOrDefault(i.get("id"), 0));
                i.put("disabledStatusNum", disabledStatusNumMap.getOrDefault(i.get("id"), 0));
            });
        }
        return result;
    }

    @Override
    public void updateSimple(JMap<String, Object> param) {

    }

    @Override
    public List<Map<String, Object>> autoSend(JMap<String, Object> param) {
        MercCouponFactory mercCouponFactory = FactoryUtils.getService(MercCouponFactory.class, "merc_coupon_base");
        return mercCouponFactory.autoSend(param);
    }

    @Override
    public void updateEnableStatus(JMap<String, Object> param) {
        LambdaUpdateWrapper<MercCoupon> lqw = new LambdaUpdateWrapper<MercCoupon>()
                .eq(MercCoupon::getId, param.getInt("id"))
                .eq(MercCoupon::getMercId, param.getInt("mercId"))
                .set(MercCoupon::getStatus, param.getBoolean("status"));
        update(lqw);
    }

}
